<?php
namespace Intergo\AuthClient;

use Intergo\AuthClient\Requests\ApiClient\DeleteApiClientRequest;
use Intergo\AuthClient\Requests\ApiClient\EnableApiClientRequest;
use Intergo\AuthClient\Requests\ApiClient\GenerateApiClientRequest;
use Intergo\AuthClient\Requests\Roles\AssignOrRevokeRequest;
use Intergo\AuthClient\Requests\Roles\DeleteRoleRequest;
use Intergo\AuthClient\Requests\Rules\AddEmailBanlistRequest;
use Intergo\AuthClient\Requests\Rules\AddEmailWhitelistRequest;
use Intergo\AuthClient\Requests\Rules\AddIpBanlistRequest;
use Intergo\AuthClient\Requests\Rules\AddIpWhitelistRequest;
use Intergo\AuthClient\Requests\Rules\GetEmailBanlistRequest;
use Intergo\AuthClient\Requests\Rules\GetEmailWhitelistRequest;
use Intergo\AuthClient\Requests\Rules\GetIpBanlistRequest;
use Intergo\AuthClient\Requests\Rules\GetIpWhitelistRequest;
use Intergo\AuthClient\Requests\Roles\GetPermissionsForUserRequest;
use Intergo\AuthClient\Requests\Roles\GetPermissionsRequest;
use Intergo\AuthClient\Requests\Roles\GetRolesForUserRequest;
use Intergo\AuthClient\Requests\Roles\GetRolesRequest;
use Intergo\AuthClient\Requests\ApiClient\GetUserApiClientRequest;
use Intergo\AuthClient\Requests\ApiKey\DeleteApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\DeleteUserApiKeysRequest;
use Intergo\AuthClient\Requests\ApiKey\DisableApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\EnableApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\GenerateApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\GetUserApiKeysRequest;
use Intergo\AuthClient\Requests\Auth\AuthenticateRequest;
use Intergo\AuthClient\Requests\Auth\InvalidateUserRequest;
use Intergo\AuthClient\Requests\Auth\LoginByRequest;
use Intergo\AuthClient\Requests\Auth\VerifyEmailRequest;
use Intergo\AuthClient\Requests\Auth\InvalidateRequest;
use Intergo\AuthClient\Requests\Auth\LoginRequest;
use Intergo\AuthClient\Requests\Auth\RefreshRequest;
use Intergo\AuthClient\Requests\Auth\RegenerateVerificationCodeRequest;
use Intergo\AuthClient\Requests\Auth\RegisterRequest;
use Intergo\AuthClient\Requests\Auth\VerifyRequest;
use Intergo\AuthClient\Requests\Roles\GetUsersRolesAndPermissionsRequest;
use Intergo\AuthClient\Requests\Roles\SearchForUserRequest;
use Intergo\AuthClient\Requests\Roles\StoreRoleRequest;
use Intergo\AuthClient\Requests\Roles\UpdateRoleRequest;
use Intergo\AuthClient\Requests\Rules\RemoveEmailBanlistRequest;
use Intergo\AuthClient\Requests\Rules\RemoveEmailWhitelistRequest;
use Intergo\AuthClient\Requests\Rules\RemoveIpBanlistRequest;
use Intergo\AuthClient\Requests\Rules\RemoveIpWhitelistRequest;
use Intergo\AuthClient\Structs\Roles\AssignOrRevoke;
use Intergo\AuthClient\Structs\Roles\DeleteRole;
use Intergo\AuthClient\Structs\Rules\GetList;
use Intergo\AuthClient\Structs\Roles\GetPermissions;
use Intergo\AuthClient\Structs\Roles\GetPermissionsForUser;
use Intergo\AuthClient\Structs\Roles\GetRoles;
use Intergo\AuthClient\Structs\Auth\LoginBy;
use Intergo\AuthClient\Structs\Roles\GetRolesForUser;
use Intergo\AuthClient\Structs\Roles\GetUsersRolesAndPermissions;
use Intergo\AuthClient\Structs\Roles\SearchForUser;
use Intergo\AuthClient\Structs\Roles\StoreRole;
use Intergo\AuthClient\Structs\Roles\UpdateRole;
use Intergo\AuthClient\Structs\Rules\UpdateEmailList;
use Intergo\AuthClient\Structs\Rules\UpdateIpList;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\AuthClient\Structs\ApiClient\DeleteApiClient;
use Intergo\AuthClient\Structs\ApiClient\DisableApiClient;
use Intergo\AuthClient\Structs\ApiClient\EnableApiClient;
use Intergo\AuthClient\Structs\ApiClient\GenerateApiClient;
use Intergo\AuthClient\Structs\ApiClient\GetUserApiClients;
use Intergo\AuthClient\Structs\ApiKey\DeleteApiKey;
use Intergo\AuthClient\Structs\ApiKey\DeleteUserApiKeys;
use Intergo\AuthClient\Structs\ApiKey\DisableApiKey;
use Intergo\AuthClient\Structs\ApiKey\EnableApiKey;
use Intergo\AuthClient\Structs\ApiKey\GenerateApiKey;
use Intergo\AuthClient\Structs\ApiKey\GetUserApiKeys;
use Intergo\AuthClient\Structs\Auth\Authenticate;
use Intergo\AuthClient\Structs\Auth\InvalidateUser;
use Intergo\AuthClient\Structs\Auth\VerifyEmail;
use Intergo\AuthClient\Structs\Auth\Invalidate;
use Intergo\AuthClient\Structs\Auth\Login;
use Intergo\AuthClient\Structs\Auth\Refresh;
use Intergo\AuthClient\Structs\Auth\RegenerateVerificationCode;
use Intergo\AuthClient\Structs\Auth\Register;
use Intergo\AuthClient\Structs\Auth\Verify;
use Intergo\BaseClient\Client;

class AuthClient extends Client
{
    /** ########### AUTHENTICATION ########### */

    /**
     * @param Register $register
     * @return BaseResponseInterface
     */
    public function register(Register $register)
    {
        $request = new RegisterRequest($this->baseUrl);
        return $this->processRequest($request, $register);
    }

    /**
     * @param Login $login
     * @return BaseResponseInterface
     */
    public function login(Login $login)
    {
        $request = new LoginRequest($this->baseUrl);
        return $this->processRequest($request, $login);
    }

    /**
     * @param LoginBy $login
     * @return BaseResponseInterface
     */
    public function loginBy(LoginBy $login)
    {
        $request = new LoginByRequest($this->baseUrl);
        return $this->processRequest($request, $login);
    }

    /**
     * @param Verify $authenticate
     * @return BaseResponseInterface
     */
    public function verify(Verify $authenticate)
    {
        $request = new VerifyRequest($this->baseUrl);
        return $this->processRequest($request, $authenticate);
    }

    /**
     * @param Authenticate $authenticate
     * @return BaseResponseInterface
     */
    public function authenticate(Authenticate $authenticate)
    {
        $request = new AuthenticateRequest($this->baseUrl);
        return $this->processRequest($request, $authenticate);
    }

    /**
     * @param Refresh $refresh
     * @return BaseResponseInterface
     */
    public function refresh(Refresh $refresh)
    {
        $request = new RefreshRequest($this->baseUrl);
        return $this->processRequest($request, $refresh);
    }

    /**
     * @param Invalidate $invalidate
     * @return BaseResponseInterface
     */
    public function invalidate(Invalidate $invalidate)
    {
        $request = new InvalidateRequest($this->baseUrl);
        return $this->processRequest($request, $invalidate);
    }

    /**
     * @param RegenerateVerificationCode $regenerateVerificationCode
     * @return BaseResponseInterface
     */
    public function regenerateVerificationCode(RegenerateVerificationCode $regenerateVerificationCode)
    {
        $request = new RegenerateVerificationCodeRequest($this->baseUrl);
        return $this->processRequest($request, $regenerateVerificationCode);
    }

    /**
     * @param VerifyEmail $verifyEmail
     * @return BaseResponseInterface
     */
    public function verifyEmail(VerifyEmail $verifyEmail)
    {
        $request = new VerifyEmailRequest($this->baseUrl);
        return $this->processRequest($request, $verifyEmail);
    }

    /**
     * @param InvalidateUser $invalidate
     * @return BaseResponseInterface
     */
    public function invalidateUser(InvalidateUser $invalidate)
    {
        $request = new InvalidateUserRequest($this->baseUrl);
        return $this->processRequest($request, $invalidate);
    }

    /** ########### API KEYS ########### */

    /**
     * @param GetUserApiKeys $getUserApiKeys
     * @return BaseResponseInterface
     */
    public function getUserApiKeys(GetUserApiKeys $getUserApiKeys)
    {
        $request = new GetUserApiKeysRequest($this->baseUrl);
        return $this->processRequest($request, $getUserApiKeys, 'GET');
    }

    /**
     * @param GenerateApiKey $generateApiKey
     * @return BaseResponseInterface
     */
    public function generateApiKey(GenerateApiKey $generateApiKey)
    {
        $request = new GenerateApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $generateApiKey);
    }

    /**
     * @param DisableApiKey $disableApiKey
     * @return BaseResponseInterface
     */
    public function disableApiKey(DisableApiKey $disableApiKey)
    {
        $request = new DisableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $disableApiKey);
    }

    /**
     * @param EnableApiKey $enableApiKey
     * @return BaseResponseInterface
     */
    public function enableApiKey(EnableApiKey $enableApiKey)
    {
        $request = new EnableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $enableApiKey);
    }

    /**
     * @param DeleteApiKey $deleteApiKey
     * @return BaseResponseInterface
     */
    public function deleteApiKey(DeleteApiKey $deleteApiKey)
    {
        $request = new DeleteApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $deleteApiKey);
    }

    /**
     * @param DeleteUserApiKeys $deleteUserApiKeys
     * @return BaseResponseInterface
     */
    public function deleteUserApiKeys(DeleteUserApiKeys $deleteUserApiKeys)
    {
        $request = new DeleteUserApiKeysRequest($this->baseUrl);
        return $this->processRequest($request, $deleteUserApiKeys);
    }

    /** ########### API CLIENTS ########### */

    /**
     * @param GetUserApiClients $getUserApiClients
     * @return BaseResponseInterface
     */
    public function getUserApiClients(GetUserApiClients $getUserApiClients)
    {
        $request = new GetUserApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $getUserApiClients, 'GET');
    }

    /**
     * @param GenerateApiClient $generateApiClient
     * @return BaseResponseInterface
     */
    public function generateApiClient(GenerateApiClient $generateApiClient)
    {
        $request = new GenerateApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $generateApiClient);
    }

    /**
     * @param DisableApiClient $disableApiClient
     * @return BaseResponseInterface
     */
    public function disableApiClient(DisableApiClient $disableApiClient)
    {
        $request = new DisableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $disableApiClient);
    }

    /**
     * @param EnableApiClient $enableApiClient
     * @return BaseResponseInterface
     */
    public function enableApiClient(EnableApiClient $enableApiClient)
    {
        $request = new EnableApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $enableApiClient);
    }

    /**
     * @param DeleteApiClient $deleteApiClient
     * @return BaseResponseInterface
     */
    public function deleteApiClient(DeleteApiClient $deleteApiClient)
    {
        $request = new DeleteApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $deleteApiClient);
    }

    /**
     * @param GetRoles $getRoles
     * @return BaseResponseInterface
     */
    public function getRoles(GetRoles $getRoles)
    {
        $request = new GetRolesRequest($this->baseUrl);
        return $this->processRequest($request, $getRoles, 'GET');
    }

    /**
     * @param GetPermissions $getPermissions
     * @return BaseResponseInterface
     */
    public function getPermissions(GetPermissions $getPermissions)
    {
        $request = new GetPermissionsRequest($this->baseUrl);
        return $this->processRequest($request, $getPermissions, 'GET');
    }

    /**
     * @param GetRolesForUser $getRoles
     * @return BaseResponseInterface
     */
    public function getRolesForUser(GetRolesForUser $getRoles)
    {
        $request = new GetRolesForUserRequest($this->baseUrl);
        return $this->processRequest($request, $getRoles, 'GET');
    }

    /**
     * @param GetPermissionsForUser $getPermissionsForUser
     * @return BaseResponseInterface
     */
    public function getPermissionsForUser(GetPermissionsForUser $getPermissionsForUser)
    {
        $request = new GetPermissionsForUserRequest($this->baseUrl);
        return $this->processRequest($request, $getPermissionsForUser, 'GET');
    }

    /**
     * @param GetUsersRolesAndPermissions $getUsersRolesAndPermissions
     * @return BaseResponseInterface
     */
    public function getUsersRolesAndPermissions(GetUsersRolesAndPermissions $getUsersRolesAndPermissions)
    {
        $request = new GetUsersRolesAndPermissionsRequest($this->baseUrl);
        return $this->processRequest($request, $getUsersRolesAndPermissions, 'GET');
    }

    /**
     * @param SearchForUser $searchForUser
     * @return BaseResponseInterface
     */
    public function searchForUser(SearchForUser $searchForUser)
    {
        $request = new SearchForUserRequest($this->baseUrl);
        return $this->processRequest($request, $searchForUser, 'GET');
    }

    /**
     * @param AssignOrRevoke $assignOrRevoke
     * @return BaseResponseInterface
     */
    public function assignOrRevokeRoles(AssignOrRevoke $assignOrRevoke)
    {
        $request = new AssignOrRevokeRequest($this->baseUrl);
        return $this->processRequest($request, $assignOrRevoke);
    }

    /**
     * @param StoreRole $storeRole
     * @return BaseResponseInterface
     */
    public function storeRole(StoreRole $storeRole)
    {
        $request = new StoreRoleRequest($this->baseUrl);
        return $this->processRequest($request, $storeRole);
    }

    /**
     * @param UpdateRole $updateRole
     * @return BaseResponseInterface
     */
    public function updateRole(UpdateRole $updateRole)
    {
        $request = new UpdateRoleRequest($this->baseUrl);
        return $this->processRequest($request, $updateRole);
    }

    /**
     * @param DeleteRole $deleteRole
     * @return BaseResponseInterface
     */
    public function deleteRole(DeleteRole $deleteRole)
    {
        $request = new DeleteRoleRequest($this->baseUrl);
        return $this->processRequest($request, $deleteRole);
    }

    /**
     * @param GetList $getBanlist
     * @return BaseResponseInterface
     */
    public function getEmailBanlist(GetList $getBanlist)
    {
        $request = new GetEmailBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $getBanlist, 'GET');
    }

    /**
     * @param GetList $getBanlist
     * @return BaseResponseInterface
     */
    public function getIpBanlist(GetList $getBanlist)
    {
        $request = new GetIpBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $getBanlist, 'GET');
    }

    /**
     * @param GetList $getBanlist
     * @return BaseResponseInterface
     */
    public function getEmailWhitelist(GetList $getBanlist)
    {
        $request = new GetEmailWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $getBanlist, 'GET');
    }

    /**
     * @param GetList $getBanlist
     * @return BaseResponseInterface
     */
    public function getIpWhitelist(GetList $getBanlist)
    {
        $request = new GetIpWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $getBanlist, 'GET');
    }

    /**
     * @param UpdateEmailList $updateEmailList
     * @return BaseResponseInterface
     */
    public function addEmailBanlist(UpdateEmailList $updateEmailList)
    {
        $request = new AddEmailBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $updateEmailList);
    }

    /**
     * @param UpdateEmailList $updateEmailList
     * @return BaseResponseInterface
     */
    public function removeEmailBanlist(UpdateEmailList $updateEmailList)
    {
        $request = new RemoveEmailBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $updateEmailList);
    }

    /**
     * @param UpdateIpList $updateIpList
     * @return BaseResponseInterface
     */
    public function addIpBanlist(UpdateIpList $updateIpList)
    {
        $request = new AddIpBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $updateIpList);
    }

    /**
     * @param UpdateIpList $updateIpList
     * @return BaseResponseInterface
     */
    public function removeIpBanlist(UpdateIpList $updateIpList)
    {
        $request = new RemoveIpBanlistRequest($this->baseUrl);
        return $this->processRequest($request, $updateIpList);
    }

    /**
     * @param UpdateEmailList $updateEmailList
     * @return BaseResponseInterface
     */
    public function addEmailWhitelist(UpdateEmailList $updateEmailList)
    {
        $request = new AddEmailWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $updateEmailList);
    }

    /**
     * @param UpdateEmailList $updateEmailList
     * @return BaseResponseInterface
     */
    public function removeEmailWhitelist(UpdateEmailList $updateEmailList)
    {
        $request = new RemoveEmailWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $updateEmailList);
    }

    /**
     * @param UpdateIpList $updateIpList
     * @return BaseResponseInterface
     */
    public function addIpWhitelist(UpdateIpList $updateIpList)
    {
        $request = new AddIpWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $updateIpList);
    }

    /**
     * @param UpdateIpList $updateIpList
     * @return BaseResponseInterface
     */
    public function removeIpWhitelist(UpdateIpList $updateIpList)
    {
        $request = new RemoveIpWhitelistRequest($this->baseUrl);
        return $this->processRequest($request, $updateIpList);
    }
}
