<?php

namespace Intergo\AuthClient\Requests;

use Intergo\BaseClient\Enums\ContentTypes;
use Intergo\BaseClient\Requests\BaseRequest;
use App\Services\Logging\GrayLoggerHandler;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Http\Message\ResponseInterface;

abstract class AuthBaseRequest extends BaseRequest
{
    /**
     * @param $token
     * @param $method
     * @param $options
     * @return mixed|ResponseInterface
     * @throws GuzzleException
     */
    public function execute($token, $method, $options)
    {
        $params = array_merge($this->authorizationsHeaders(), $options);
        if (isset($options['json'])) {
            $response = $this->send($method, $params);
            return json_decode($response->getBody()->getContents(), true);
        }
        return $this->send($method, $params);
    }

    /**
     * @return array[]
     */
    protected function authorizationsHeaders()
    {
        return [
            'headers' => [
                'Accept' => ContentTypes::APPLICATION_JSON,
                'Content-Type' => ContentTypes::APPLICATION_JSON,
                'X-TRACE-ID' => app(GrayLoggerHandler::class)->getTraceId()
            ]
        ];
    }
}
